import UIKit
import BouncyLayout
import IGListKit

class ViewController: UIViewController {
    lazy var adapter: ListAdapter = {
        return ListAdapter(updater: ListAdapterUpdater(), viewController: self, workingRangeSize: 0)
    }()

    let numberOfItems = 100
    var randomCellStyle: CellStyle { return CellStyle(example: self.example) }

    lazy var style: [CellStyle] = { (0..<self.numberOfItems).map { _ in self.randomCellStyle } }()
    lazy var topOffset: [CGFloat] = { (0..<self.numberOfItems).map { _ in CGFloat(arc4random_uniform(250)) } }()

    var insets: UIEdgeInsets {
        guard let example = self.example else { return .zero }

        switch example {
        case .chatMessage: return UIEdgeInsets(top: 200, left: 0, bottom: 200, right: 0)
        case .photosCollection: return UIEdgeInsets(top: 200, left: 0, bottom: 200, right: 0)
        case .barGraph: return UIEdgeInsets(top: 0, left: 200, bottom: 0, right: 200)
        }
    }

    var additionalInsets: UIEdgeInsets {
        guard let example = self.example else { return .zero }

        switch example {
        case .chatMessage: return UIEdgeInsets(top: 5, left: 0, bottom: 5, right: 0)
        case .photosCollection: return UIEdgeInsets(top: 15, left: 15, bottom: 15, right: 15)
        case .barGraph: return UIEdgeInsets(top: 0, left: 0, bottom: 0, right: 10)
        }
    }

    lazy var layout = BouncyLayout(style: .regular)

    lazy var collectionView: UICollectionView = {

        if self.example == .barGraph {
            self.layout.scrollDirection = .horizontal
        }

        let view = UICollectionView(frame: .zero, collectionViewLayout: self.layout)
//       FIXME : No problem with normal UICollectionViewFlowLayout// let view = UICollectionView(frame: .zero, collectionViewLayout: UICollectionViewFlowLayout())
        view.translatesAutoresizingMaskIntoConstraints = false
        view.delaysContentTouches = false
        view.alwaysBounceVertical = self.example != .barGraph
        view.alwaysBounceHorizontal = self.example == .barGraph
        view.backgroundColor = nil
        view.isOpaque = false

        return view
    }()

    var example: Example!

    convenience init(example: Example) {
        self.init()
        self.example = example
    }

    override func viewDidLoad() {
        super.viewDidLoad()

        adapter.collectionView = collectionView
        adapter.dataSource = self

        view.backgroundColor = .white
        view.clipsToBounds = true

        collectionView.contentInset = UIEdgeInsets(top: insets.top + additionalInsets.top, left: insets.left + additionalInsets.left, bottom: insets.bottom + additionalInsets.bottom, right: insets.right + additionalInsets.right)
        collectionView.scrollIndicatorInsets = UIEdgeInsets(top: insets.top, left: insets.left, bottom: insets.bottom, right: insets.right)
        view.addSubview(collectionView)

        NSLayoutConstraint.activate([
            collectionView.topAnchor.constraint(equalTo: view.topAnchor, constant: -insets.top),
            collectionView.leftAnchor.constraint(equalTo: view.leftAnchor, constant: -insets.left),
            collectionView.bottomAnchor.constraint(equalTo: view.bottomAnchor, constant: insets.bottom),
            collectionView.rightAnchor.constraint(equalTo: view.rightAnchor, constant: insets.right)
        ])
    }
}

extension ViewController: ListAdapterDataSource, ListDisplayDelegate, CellDelegate {
    func objects(for listAdapter: ListAdapter) -> [ListDiffable] {
        return style as [ListDiffable]
    }

    func listAdapter(_ listAdapter: ListAdapter, sectionControllerFor object: Any) -> ListSectionController {
        let sectionController = SectionController(example: example)
        sectionController.delegate = self
        return sectionController
    }

    func emptyView(for listAdapter: ListAdapter) -> UIView? {
        return nil
    }

    func listAdapter(_ listAdapter: ListAdapter, willDisplay sectionController: ListSectionController, cell: UICollectionViewCell, at index: Int) {
        guard let cell = cell as? Cell else { return }
        cell.setCell(style: style[index], example: example)

        if example == .barGraph {
            cell.top.constant = topOffset[index]
        }
    }

    func listAdapter(_ listAdapter: ListAdapter, willDisplay sectionController: ListSectionController) {
    }

    func listAdapter(_ listAdapter: ListAdapter, didEndDisplaying sectionController: ListSectionController) {
    }

    func listAdapter(_ listAdapter: ListAdapter, didEndDisplaying sectionController: ListSectionController, cell: UICollectionViewCell, at index: Int) {
    }
    
    func didSelect(_ style: CellStyle) {
//        adapter.performUpdates(animated: true, completion: nil)
        // This has to be used to turn the cell into red, not sure it's normal
        adapter.reloadObjects([style])
    }
}

protocol CellDelegate: class {
    func didSelect(_ style: CellStyle)
}

class SectionController: ListSectionController {
    var style: CellStyle!
    var example: Example!
    weak var delegate: CellDelegate?

    init(example: Example) {
        super.init()
        self.example = example
        minimumInteritemSpacing = example == .photosCollection ? 15 : 0
        minimumLineSpacing = example == .photosCollection ? 15 : 0
    }

    override func numberOfItems() -> Int {
        return 1
    }

    override func cellForItem(at index: Int) -> UICollectionViewCell {
        let cell = collectionContext!.dequeueReusableCell(of: Cell.self, for: self, at: index) as! Cell
        cell.setCell(style: style, example: example)
        return cell
    }

    override func didUpdate(to object: Any) {
        self.style = object as! CellStyle
    }

    override func didSelectItem(at index: Int) {
        style.style = .red
        style.size = CGSize(width: style.size.width, height: 300)
        UIView.animate(withDuration: 2, delay: 0, usingSpringWithDamping: 0.6, initialSpringVelocity: 0.5, options: .curveEaseInOut, animations: {
            self.collectionContext?.invalidateLayout(for: self, completion: nil)
        }, completion: nil)
        
        delegate?.didSelect(style)
    }

    override func sizeForItem(at index: Int) -> CGSize {
        return style.size
    }
}
